/*
MIDI Sequencer V.1.0
(C)2007 by Lindenberg Electronics
All Rights Reserved
*/

#include "main.h"
#include <avr/io.h>
#include <stdint.h>
#include <util/delay.h>
#include "hd44780.h"
#include "uart.h"


/* PORT defines */
#define _KEYPORT	PORTD
#define _KEYOUT		DDRD	

/* keys from keyped */
#define _KEY_LEFT	0b00010000
#define _KEY_RIGHT	0b00001000
#define _KEY_UP		0b00000100

/* main loop msg */
#define _MSG_RESET		0x00
#define _MSG_NOTEPLAY	0x05

/* notes */
#define _N_BLANK		0xFF
#define _N_C			0x00
#define _N_CM			0x01
#define _N_D			0x02
#define _N_DM			0x03
#define _N_E			0x04
#define _N_F			0x05
#define _N_FM			0x06
#define _N_G			0x07
#define _N_GM			0x08
#define _N_A			0x09
#define _N_AM			0x0A
#define _N_H			0x0B	


/* ADC */
#define _ADCFR			0x40;

/* generel MIDI commands (all on channel 1)*/
#define _MIDI_NOTEON	0x90
#define _MIDI_NOTEOFF	0x80
#define _MIDI_VELOCITY	0x7F
#define _MIDI_C3		0x30
#define _MIDI_CLOCK		0xF8
#define _MIDI_CONTROLER 0xB0
#define _MIDI_CUTOFF  	0x4A


/* adc channel struct */

struct _adc_data {
	uint16_t channel[8]; 	/* adc channels */
	uint16_t buffer[8];  	/* buffer channels */
	uint8_t data_ready;     /* channel convention compleate */
} adc_data;


/* global vars */
volatile uint8_t _msg = _MSG_RESET;
volatile uint8_t _nc, _adcc = 0;
volatile uint16_t time_interval;
volatile struct adc_data;


void enable_timer(uint8_t bpm)
{
	TCCR1B = 0b00000011;   						/* prescaler = 64 */
	time_interval = (F_CPU/64) / (4*bpm/60);	/* calculate timer */
	OCR1A = time_interval;	
	TIMSK |= (1<<OCIE1A);
}


void enable_adc(void)
{
	/* prescaler = 1M; freerunning = on; interrupt = on; adc = on */
	SFIOR |= (0<<ADTS2) | (0<<ADTS1) | (0<<ADTS0);
	ADCSRA |= (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0) | (1<<ADEN) | (1<<ADIE) | (1<<ADATE);
	/* start with first channel */
	ADMUX = 0 + _ADCFR;
}


int main(void)
{
	uint8_t notes[16] ={_N_C, _N_D, _N_E, _N_F, _N_G, _N_A, _N_H, _N_C, 
						_N_C, _N_C, _N_C, _N_C, _N_C, _N_C, _N_C, _N_C};

	uint8_t last_note = _N_BLANK;

	lcd_init();
	lcd_clear();
	lcd_home();

	uart_enable();
	uart_putstr("test adc");

	lcd_putstr(" MIDI-Sequencer");
	lcd_setline(2);
	lcd_putstr(" (C)2007 by LE");

//	enable_timer(135);			// first init all, then enable int's
	enable_adc();
	
	sei();

	ADCSRA |= (1<<ADSC); /* start first adc convention */

	adc_data.data_ready = 0;

	/* main loop */
	while(1) {	
	
	/* test adc */
	uart_putc(adc_data.channel[0]);

		/* main msg handler */
/*		switch(_msg) {
		
			case _MSG_NOTEPLAY: 
				if (notes[_nc] != _N_BLANK) {
					uart_putc( _MIDI_NOTEOFF );
					uart_putc( _MIDI_C3 + last_note );
					uart_putc( 0x00 );
		
					uart_putc( _MIDI_NOTEON );
					uart_putc( _MIDI_C3 + notes[_nc] );
					uart_putc( _MIDI_VELOCITY );

					last_note = notes[_nc];
				}
		
				_msg = _MSG_RESET;
				break;							// sicherheithalber, vergisst man gern beim erweitern des SWITCH
			default: break;
		}
		/* main msg handler */
	
	}

	return 0;
}


/* timer interrupt for bmp calc */ 
ISR(TIMER1_COMPA_vect)
{
	OCR1A += time_interval;

	if(_nc<15) _nc++; else _nc=0;
	_msg = _MSG_NOTEPLAY;
}


/* adc complete interrupt */
ISR(ADC_vect)
{
	adc_data.channel[_adcc] = ADC;
	

	if(_adcc<7) _adcc++; 
	else {
		_adcc = 0;
		adc_data.data_ready = 1;
	}

	ADMUX = _adcc + _ADCFR;
}


